# !/usr/bin/python
# coding=utf-8
import inspect
import importlib
import pkgutil


__package__ = "mayatk"
__version__ = '0.6.2'


# Define dictionaries to map class names, method names, class method names, and sub-modules to their respective modules
CLASS_TO_MODULE = {}
METHOD_TO_MODULE = {}
CLASS_METHOD_TO_MODULE = {}
SUBMODULE_TO_MODULE = {}

# Build the dictionaries by iterating over all submodules and sub-packages of the package
for importer, modname, ispkg in pkgutil.walk_packages(__path__, __name__ + "."):
    module = importlib.import_module(modname)
    for name, obj in module.__dict__.items():
        if inspect.isclass(obj):
            CLASS_TO_MODULE[obj.__name__] = modname
            for method_name, method_obj in inspect.getmembers(
                obj, predicate=inspect.isfunction
            ):
                METHOD_TO_MODULE[method_name] = (modname, obj.__name__)
            for method_name, method_obj in inspect.getmembers(
                obj, predicate=inspect.ismethod
            ):
                CLASS_METHOD_TO_MODULE[method_name] = (modname, obj.__name__)
    if not ispkg:
        submodule_name = modname.split(".")[-1]
        SUBMODULE_TO_MODULE[submodule_name] = modname

# Define a dictionary to store imported module objects
IMPORTED_MODULES = {}


def __getattr__(name):
    # Check if the requested attribute is a sub-module we need to import
    if name in SUBMODULE_TO_MODULE:
        module_name = SUBMODULE_TO_MODULE[name]
        if module_name not in IMPORTED_MODULES:
            # If the module hasn't been imported yet, import it and add it to the dictionary
            module = importlib.import_module(module_name)
            IMPORTED_MODULES[module_name] = module
        else:
            module = IMPORTED_MODULES[module_name]
        # Return the requested sub-module object from the module
        return module

    # Check if the requested attribute is a class we need to import
    if name in CLASS_TO_MODULE:
        module_name = CLASS_TO_MODULE[name]
        if module_name not in IMPORTED_MODULES:
            # If the module hasn't been imported yet, import it and add it to the dictionary
            module = importlib.import_module(module_name)
            IMPORTED_MODULES[module_name] = module
        else:
            module = IMPORTED_MODULES[module_name]
        # Return the requested class object from the module
        return getattr(module, name)

    # Check if the requested attribute is a method we need to import
    elif name in METHOD_TO_MODULE:
        module_name, class_name = METHOD_TO_MODULE[name]
        if module_name not in IMPORTED_MODULES:
            # If the module hasn't been imported yet, import it and add it to the dictionary
            module = importlib.import_module(module_name)
            IMPORTED_MODULES[module_name] = module
        else:
            module = IMPORTED_MODULES[module_name]
        # Get the class object and return the requested method object from it
        class_obj = getattr(module, class_name)
        return getattr(class_obj, name)

    # Check if the requested attribute is a class method we need to import
    elif name in CLASS_METHOD_TO_MODULE:
        module_name, class_name = CLASS_METHOD_TO_MODULE[name]
        if module_name not in IMPORTED_MODULES:
            # If the module hasn't been imported yet, import it and add it to the dictionary
            module = importlib.import_module(module_name)
            IMPORTED_MODULES[module_name] = module
        else:
            module = IMPORTED_MODULES[module_name]
        # Get the class object and return the requested class method object from it
        class_obj = getattr(module, class_name)
        return getattr(class_obj, name)

    # If the requested attribute is not a class, method, class method, or sub-module we handle, raise an AttributeError
    raise AttributeError(f"module {__package__} has no attribute '{name}'")


# --------------------------------------------------------------------------------------------


# --------------------------------------------------------------------------------------------
# Notes
# --------------------------------------------------------------------------------------------


# --------------------------------------------------------------------------------------------
# deprecated:
# --------------------------------------------------------------------------------------------


# def __getattr__(attr):
# 	"""This function dynamically imports a module and returns an attribute from the module.

# 	Parameters:
# 		attr (str): The name of the attribute to be imported. The name should be in the format
# 					'module_name.attribute_name' or just 'attribute_name'.
# 	Returns:
# 		(obj) The attribute specified by the `attr` argument.

# 	:Raises:
# 		AttributeError: If the specified attribute is not found in either the original module
# 						or the 'Misc' module within the package.
# 	Example:
# 		<package>.__getattr__('module1.attribute1') #returns: <attribute1 value>
# 		<package>.__getattr__('attribute1') #returns: <attribute1 value>
# 	"""
# 	try:
# 		module = __import__(f"{__package__}.{attr}", fromlist=[f"{attr}"])
# 		setattr(sys.modules[__name__], attr, getattr(module, attr))
# 		return getattr(module, attr)

# 	except (ValueError, ModuleNotFoundError):
# 		module = __import__(f"{__package__}.Misc", fromlist=["Misc"])
# 		return getattr(module, attr)

# 	except AttributeError as error:
# 		raise AttributeError(f"Module '{__package__}' has no attribute '{attr}'") from error



# !/usr/bin/python
# coding=utf-8
try:
    import pymel.core as pm
except ImportError as error:
    print(__file__, error)

# from this package:
from mayatk import misc_utils


class Cam(object):
    """ """

    @staticmethod
    @misc_utils.Misc.undo
    def group_cameras(
        name="cameras", non_default=True, root_only=False, hide_group=False
    ):
        """Group cameras in the scene based on the provided parameters.

        Parameters:
                name (str): The name of the group that will contain the cameras. Default is 'cameras'.
                non_default (bool, optional): If True, only non-default cameras will be grouped. Default is True.
                root_only (bool, optional): If True, only root-level cameras (not parented to other objects) will be grouped. Default is False.
                hide_group (bool, optional): If True, the newly created group will be hidden. Default is False.

        Returns:
                PyNode: The created group node containing the cameras.
        """
        if pm.objExists(name):  # Check if the group already exists
            pm.error(f"Group '{name}' already exists.")
            return

        # Create the group and set it's visibility
        group = pm.group(empty=True, name=name)
        group.visibility.set(not hide_group)

        default_cameras = (
            "side",
            "front",
            "top",
            "persp",
            "back",
            "bottom",
            "left",
            "alignToPoly",
        )  # List of default cameras
        all_cameras = pm.ls(type="camera")  # Get all cameras in the scene
        all_camera_transforms = [
            cam.get_parent() for cam in all_cameras
        ]  # Get the parent transform nodes of the cameras

        if root_only:  # Filter cameras based on the root_only flag
            all_camera_transforms = [
                cam for cam in all_camera_transforms if cam.get_parent() is None
            ]

        # Parent cameras to the group based on the non_default flag
        for cam in all_camera_transforms:
            if non_default:
                if not any(
                    [cam.name().endswith(def_cam) for def_cam in default_cameras]
                ):
                    pm.parent(cam, group)
            else:
                pm.parent(cam, group)

        return group

    @classmethod
    def toggle_safe_frames(cls):
        """Toggle display of the film gate for the current camera."""
        camera = cls.get_current_cam()

        state = pm.camera(camera, q=True, displayResolution=1)
        if state:
            pm.camera(
                camera,
                edit=1,
                displayFilmGate=False,
                displayResolution=False,
                overscan=1.0,
            )
        else:
            pm.camera(
                camera,
                edit=1,
                displayFilmGate=False,
                displayResolution=True,
                overscan=1.3,
            )

    @staticmethod
    def get_current_cam():
        """Get the currently active camera."""
        from maya.OpenMaya import MDagPath
        from maya.OpenMayaUI import M3dView

        view = M3dView.active3dView()
        cam = MDagPath()
        view.getCamera(cam)
        camPath = cam.fullPathName()
        return camPath

    @staticmethod
    @misc_utils.Misc.undo
    def create_camera_from_view(name="camera#"):
        """Create a new camera based on the current view."""
        # Find the current modelPanel (viewport)
        current_panel = None
        for panel in misc_utils.get_panel(all=True):
            if misc_utils.get_panel(typeOf=panel) == "modelPanel":
                current_panel = panel
                break

        if current_panel:
            if misc_utils.get_panel(typeOf=current_panel) == "modelPanel":
                camera = pm.modelPanel(current_panel, q=1, cam=1)
                new_camera = pm.duplicate(camera)[0]
                pm.showHidden(new_camera)
                new_camera = pm.rename(new_camera, name)
                print(f"# Result: {new_camera} #")
                return new_camera
        else:
            print("No modelPanel found")


# --------------------------------------------------------------------------------------------

if __name__ == "__main__":
    pass

# --------------------------------------------------------------------------------------------
# Notes
# --------------------------------------------------------------------------------------------


# --------------------------------------------------------------------------------------------
# deprecated:
# --------------------------------------------------------------------------------------------

# @classmethod
#   def matchTransformByVertexOrder(cls, source, target):
#       '''Match transform and rotation on like objects by using 3 vertices from each object.
#       The vertex order is transferred to the target object(s).

#       Parameters:
#           source (str/obj): The object to move from.
#           target (str/obj): The object to move to.
#       '''
#       pm.polyTransfer(source, alternateObject=target, vertices=2) #vertices positions are copied from the target object.

#       source_verts = [pm.ls(source, objectsOnly=1)[0].verts[i] for i in range(3)]
#       target_verts = [pm.ls(target, objectsOnly=1)[0].verts[i] for i in range(3)]

#       cls.align_using_three_points(source_verts+target_verts)

# @staticmethod
#   def getComponentPoint(component, alignToNormal=False):
#       '''Get the center point from the given component.

#       Parameters:
#           component (str/obj): Object component.
#           alignToNormal (bool): Constain to normal vector.

#       Returns:
#           (tuple) coordinate as xyz float values.
#       '''
#       if ".vtx" in str(component):
#           x = pm.polyNormalPerVertex(component, q=True, x=1)
#           y = pm.polyNormalPerVertex(component, q=True, y=1)
#           z = pm.polyNormalPerVertex(component, q=True, z=1)
#           xyz = [sum(x) / float(len(x)), sum(y) / float(len(y)), sum(z) / float(len(z))] #get average

#       elif ".e" in str(component):
#           componentName = str(component).split(".")[0]
#           vertices = pm.polyInfo (component, edgeToVertex=1)[0]
#           vertices = vertices.split()
#           vertices = [componentName+".vtx["+vertices[2]+"]",componentName+".vtx["+vertices[3]+"]"]
#           x=[];y=[];z=[]
#           for vertex in vertices:
#               x_ = pm.polyNormalPerVertex (vertex, q=True, x=1)
#               x.append(sum(x_) / float(len(x_)))
#               y_ = pm.polyNormalPerVertex (vertex, q=True, y=1)
#               x.append(sum(y_) / float(len(y_)))
#               z_ = pm.polyNormalPerVertex (vertex, q=True, z=1)
#               x.append(sum(z_) / float(len(z_)))
#           xyz = [sum(x) / float(len(x)), sum(y) / float(len(y)), sum(z) / float(len(z))] #get average

#       else:# elif ".f" in str(component):
#           xyz = pm.polyInfo (component, faceNormals=1)
#           xyz = xyz[0].split()
#           xyz = [float(xyz[2]), float(xyz[3]), float(xyz[4])]

#       if alignToNormal: #normal constraint
#           normal = pm.mel.eval("unit <<"+str(xyz[0])+", "+str(xyz[1])+", "+str(xyz[2])+">>;") #normalize value using MEL
#           # normal = [round(i-min(xyz)/(max(xyz)-min(xyz)),6) for i in xyz] #normalize and round value using python

#           constraint = pm.normalConstraint(component, object_,aimVector=normal,upVector=[0,1,0],worldUpVector=[0,1,0],worldUpType="vector") # "scene","object","objectrotation","vector","none"
#           pm.delete(constraint) #orient object_ then remove constraint.

#       vertexPoint = pm.xform (component, q=True, translation=1) #average vertex points on destination to get component center.
#       x = vertexPoint[0::3]
#       y = vertexPoint[1::3]
#       z = vertexPoint[2::3]

#       return tuple(round(sum(x) / float(len(x)),4), round(sum(y) / float(len(y)),4), round(sum(z) / float(len(z)),4))



# !/usr/bin/python
# coding=utf-8
import os, sys
import unittest
import inspect

import pymel.core as pm

from mayatk import Rig


# sfr = pm.melGlobals['cmdScrollFieldReporter']
# pm.cmdScrollFieldReporter(sfr, edit=1, clear=1)


class Main(unittest.TestCase):
    """Main test class."""

    def perform_test(self, cases):
        """Execute the test cases."""
        for case in cases:
            if isinstance(case, str):
                expression = case
                expected_result = cases[case]
                method_name = str(expression).split("(")[0]
            else:
                result, expected_result = case
                method_name = result.__class__.__name__
                expression = None

            try:
                path = os.path.abspath(inspect.getfile(eval(method_name)))
            except (TypeError, IOError):
                path = ""

            if expression:
                result = eval(expression)

            self.assertEqual(
                result,
                expected_result,
                f"\n\n# Error: {path}\n#\tCall: {method_name}({', '.join(map(str, function_args)) if 'function_args' in locals() else ''})\n#\tExpected {type(expected_result)}: {expected_result}\n#\tReturned {type(result)}: {result}",
            )

    @staticmethod
    def replace_mem_address(obj):
        """Replace memory addresses in a string representation of an object with a fixed format of '0x00000000000'.

        Parameters:
                obj (object): The input object. The function first converts this object to a string using the `str` function.

        Returns:
                (str) The string representation of the object with all memory addresses replaced.

        Example:
                >>> replace_mem_address("<class 'str'> <PySide2.QtWidgets.QWidget(0x1ebe2677e80, name='MayaWindow') at 0x000001EBE6D48500>")
                "<class 'str'> <PySide2.QtWidgets.QWidget(0x00000000000, name='MayaWindow') at 0x00000000000>"
        """
        import re

        return re.sub(r"0x[a-fA-F\d]+", "0x00000000000", str(obj))


class Rig_test(Main, Rig):
    """ """

    # Tear down the any previous test by creating a new scene:
    pm.mel.file(new=True, force=True)

    # assemble the test scene:
    if not pm.objExists("loc"):
        loc = pm.spaceLocator(name="loc")

    if not pm.objExists("cyl"):
        cyl = pm.polyCylinder(
            radius=5,
            height=10,
            subdivisionsX=6,
            subdivisionsY=1,
            subdivisionsZ=1,
            name="cyl",
        )

    def test_createLocator(self):
        """ """
        self.perform_test(
            {
                "self.create_locator('_loc')": "_loc",
            }
        )

    def test_removeLocator(self):
        """ """
        self.perform_test(
            {
                "self.remove_locator('loc')": None,
            }
        )

    def test_resetPivotTransforms(self):
        """ """
        self.perform_test(
            {
                "self.reset_pivot_transforms('cyl')": None,
            }
        )

    def test_bakeCustomPivot(self):
        """ """
        self.perform_test(
            {
                "self.bake_custom_pivot('cyl')": None,
                "self.bake_custom_pivot('cyl', position=True)": None,
                "self.bake_custom_pivot('cyl', orientation=True)": None,
            }
        )

    def test_setAttrLockState(self):
        """ """
        self.perform_test(
            {
                "self.set_attr_lock_state('cyl')": None,
            }
        )

    def test_createGroup(self):
        """ """
        self.perform_test(
            {
                "self.create_group(name='emptyGrp').name()": "emptyGrp",
            }
        )

    def test_createGroupLRA(self):
        """ """
        self.perform_test(
            {
                "self.create_group_with_first_obj_lra('cyl', 'LRAgrp').name()": "LRAgrp",
            }
        )

    def test_createLocatorAtObject(self):
        """ """
        self.perform_test(
            {
                "self.create_locator_at_object('cyl')": None,
            }
        )


# -----------------------------------------------------------------------------

if __name__ == "__main__":
    unittest.main(exit=False)


# -----------------------------------------------------------------------------
# Notes
# -----------------------------------------------------------------------------

# """
# def test_(self):
#   '''
#   '''
#   self.perform_test({
#       # "self.": '',
#   })


# def test_(self):
#   '''
#   '''
#   self.perform_test({
#       # "self.": '',
#   })


# def test_(self):
#   '''
#   '''
#   self.perform_test({
#       # "self.": '',
#   })


# def test_(self):
#   '''
#   '''
#   self.perform_test({
#       # "self.": '',
#   })
# """

# # Deprecated ---------------------



import os
import setuptools

from mayatk import __package__, __version__
from pythontk import File, Str


long_description = File.get_file_contents("docs/README.md")
description = Str.get_text_between_delimiters(
    long_description,
    "<!-- short_description_start -->",
    "<!-- short_description_end -->",
    as_string=True,
)

setuptools.setup(
    name=__package__,
    version=__version__,
    author="Ryan Simpson",
    author_email="m3trik@outlook.com",
    description=description,
    long_description=long_description,
    long_description_content_type="text/markdown",
    url=f"https://github.com/m3trik/{__package__}",
    packages=setuptools.find_packages(exclude=["pymel", "pymel.*"]),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    include_package_data=True,
    data_files=File.get_dir_contents(
        __package__, "filepaths", exc_files=["*.py", "*.pyc", "*.json"]
    ),  # ie. ('uitk/ui/0', ['uitk/ui/0/init.ui']),
)

# --------------------------------------------------------------------------------------------


# --------------------------------------------------------------------------------------------
# Notes
# --------------------------------------------------------------------------------------------


# Deprecated ------------------------------------


# def gen_data_files(dirs, exc=[], inc=[]):
# 	'''
# 	'''
# 	dirs = Iter.make_iterable(dirs)
# 	exc = Iter.make_iterable(exc)
# 	inc = Iter.make_iterable(inc)

# 	results = []
# 	for src_dir in dirs:
# 		for root, dirs, files in os.walk(src_dir):
# 			filtered=[]
# 			for f in files:
# 				ext = File.format_path(f, 'ext')
# 				if f in exc or '*.'+ext in exc:
# 					continue
# 				if any(inc): #filter inc for None values so not to get a false positive.
# 					if not f in inc and not '*.'+ext in inc:
# 						continue
# 				filtered.append(f)

# 			if filtered:
# 				results.append((root, list(map(lambda f:root + "/" + f, filtered))))
# 	return results

# # for i in gen_data_files('uitk', exc=['*.py', '*.pyc', '*.json']):
# 	# print (i)
